# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .onboarding_option import OnboardingOption
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class CommonOnboardingOptions(UniversalBaseModel):
    terms_of_service: OnboardingOption = pydantic.Field(alias="termsOfService")
    email: OnboardingOption
    name: OnboardingOption
    address: OnboardingOption
    phone: OnboardingOption
    ten_ninety_nine: OnboardingOption = pydantic.Field(alias="tenNinetyNine")
    w_9: OnboardingOption = pydantic.Field(alias="w9")
    bank_statement: OnboardingOption = pydantic.Field(alias="bankStatement")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
