# This file was auto-generated by Fern from our API Definition.

import typing

DocumentType = typing.Union[
    typing.Literal["INVOICE", "TEN_NINETY_NINE", "W9", "CHECK", "BANK_STATEMENT", "OTHER"], typing.Any
]
