# This file was auto-generated by Fern from our API Definition.

from .transaction_response_base import TransactionResponseBase
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class TransactionResponseBankToMailedCheckBase(TransactionResponseBase):
    check_number: int = pydantic.Field(alias="checkNumber")
    """
    The number of the check
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
