# This file was auto-generated by Fern from our API Definition.

from .common_onboarding_options import CommonOnboardingOptions
from .onboarding_option import OnboardingOption
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class IndividualOnboardingOptions(CommonOnboardingOptions):
    date_of_birth: OnboardingOption = pydantic.Field(alias="dateOfBirth")
    ssn: OnboardingOption

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
