# This file was auto-generated by Fern from our API Definition.

import typing

TransactionStatus = typing.Union[
    typing.Literal["CREATED", "PENDING", "COMPLETED", "FAILED", "REVERSED", "QUEUED", "CANCELED"], typing.Any
]
