# This file was auto-generated by Fern from our API Definition.

import typing

TransactionType = typing.Union[
    typing.Literal["bankAccountToBankAccount", "bankAccountToMailedCheck", "custom", "offPlatform"], typing.Any
]
